#include maps\_utility;
#include common_scripts\utility;
#include maps\omahabeach_code;
#include maps\omahabeach_util;

/***********************************************************************																
		
		Omaha Beach
			Mapping & Scripting: Alex 'Sparks' Romo
		
		TODO:
			1) Add in ambient packages.
					- Ocean waves / Yelling / Gunshots / etc...
			2) Spread out drone spawn.
					- Too many spawning in = Too many new entities in snapshot.
			3) Introscreen should take 58 seconds to time perfectly with music.
			4) Bunker finale fade into is too sudden. Not timed right.
			5) Multi-Language support.
		
		Known Bugs:
			1) M1Garand disappears on drones due to LOD.
					- Possibly edit the LOD?
			2) Latter intro texts don't fade nicely; snap quickly into destroy().
					- Don't know why yet, modulated the process and only works on
						first one. :(
			3) Load video doesn't work from *_load.FF
					- Don't really want to have users manually put in video/main, meh.
			4) During bunker finale, player can see outside left area of bunker.
					- Looks bland.
			
		Notes:
			1) Tweak mortars; feel icky at the moment.
					- Add water ones too?
			2) devCheck() allows me to skip/execute parts of the script without
				 having to use start_points but rather Dvars.
				 	- "developer" - "1"				: This needs to be set.
				 	- "skip_intro" - "1"			: Skip intro text.
				 	- "free_me" - "1"					: Doesn't force player onto MG42.
				 	- "dontQuitMap" - "1"			: Don't quit the map after bunker finale.
				 	- "deadAnimCheck" - "1"		: Preview the dead drone animations.
				 	- "rideAnimCheck" - "1"		: Preview the higgin rider drone animations.
			3) level.player == First player to connect.
					- This is SP map only.
			

***********************************************************************/

main()
{
	/*--------------------
	 REFLECTIONS
	----------------------*/
	if ( getdvar( "r_reflectionProbeGenerate" ) == "1" )
		return;
	
	/*--------------------
	 NETWORKING
	----------------------*/
	level thread onPlayerConnect();
	
	/*--------------------
	 LEVEL VARIABLES
	----------------------*/
	levelVariables();
	
	/*--------------------
	 FLAGS
	----------------------*/
	flagInits();
	
	/*--------------------
	 START POINTS
	----------------------*/
	default_start( ::map_startpoint );
	add_start( "default", ::map_startpoint, &"OMAHABEACH_START_DEFAULT" );	
	add_start( "bunker", ::bunker_startPoint, &"OMAHABEACH_START_BUNKER" );	
	
	/*--------------------
	 PRECACHE
	----------------------*/
	preCacheStuff();
	
	/*--------------------
	 SCRIPT CALLS
	----------------------*/
	scriptCalls();

	/*--------------------
	 DRONES
	----------------------*/
	level.drone_spawnFunction[ "allies" ] = character\char_usa_marine_r_rifle::main; 
	maps\_drones::init();
	
	/*--------------------
	 LEVEL FX
	----------------------*/
	maps\omahabeach_fx::main();
	
	/*--------------------
	 THREAD BIAS GROUP
	----------------------*/
	CreateThreatBiasGroup( "player" );
	CreateThreatBiasGroup( "beachAI" );
	CreateThreatBiasGroup( "officerAI" );
	CreateThreatBiasGroup( "flameThrowerAI" );
	SetThreatBias( "flameThrowerAI", "player", 0 );
	SetThreatBias( "flameThrowerAI", "officerAI", 1000 );
	SetThreatBias( "beachAI", "player", 1 );
	
	/*--------------------
	 LOAD
	----------------------*/
	maps\_load::main();
	
	/*--------------------
	 AI TEAM HACK
	----------------------*/
	array_thread( GetSpawnerTeamArray( "axis" ),			:: add_spawn_function, ::myTeamHack );
	array_thread( GetSpawnerTeamArray( "allies" ),		:: add_spawn_function, ::theirTeamHack );
	//battlechatter_off( "allies" );
	//battlechatter_off( "axis" );
	
	/*--------------------
	 EARLY LEVEL FIX
	----------------------*/
	level.early_level[ level.script ] = false;
	
	/*--------------------
	 AMBIENT SUPPORT
	----------------------*/
	level thread maps\omahabeach_amb::main();
	
	/*--------------------
	 ANIMATION SUPPORT
	----------------------*/
	maps\omahabeach_anim::main();
	
	/*--------------------
	 PLAYER CONNECT
	----------------------*/
	flag_wait( "all_players_connected" );
	level.player = get_players()[0];
	assertEx( isDefined( level.player ), "Player does not exist." );
}