#include maps\_utility;
#include common_scripts\utility;
#include maps\_anim;

/***********************************************************************
	DEV
***********************************************************************/

devCheck( dvar, type, value )
{
	if( getDvarInt( "developer" ) < 1 )
	{
		return false;
	}
	
	if( isDefined( type ) && type == "string" )
	{
		assert( value );
		
		if( getDvar( dvar ) == value )
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	else if( isDefined( type ) && type == "float" )
	{
		assert( value );
		
		if( getDvarFloat( dvar ) == value )
		{
			return true;
		}
		else
		{
			return false;
		}
	} 
	else
	{
		if( !isDefined( value ) )
		{
			value = 1;
		}
		
		if( getDvarInt( dvar ) == value )
		{
			return true;
		}
		else
		{
			return false;
		}
	}
}

animPreviewer( animArray, animTree )
{
	dummy = spawn( "script_model" , groundpos( level.player.origin ) );
	dummy character\char_usa_marine_r_nb_rifle:: main();
	dummy animTreeInit( animTree );
	
	wait ( 1 );
	
	for( x = 0 ; x < animArray.size ; x++ )
	{
		iPrintLnBold( "Animation # Played: " + x );
		
		dummy setFlaggedAnimKnob( "idle", animArray[ x ], 1, .1, 1 ); 
		
		wait 4;
	}
	
	wait ( 1 );
	
	dummy delete();
}

/***********************************************************************
	HUD
***********************************************************************/

titleCreate( text, direction, moveTime, fadeTime )
{
	assertEx( isDefined( text ), "No title text defined." );
	if( !isDefined( direction ) )
	{
		printLn( "Title direction not defined. Intended?" );
		direction = "left";
	}
	if( !isDefined( moveTime ) )
	{
		moveTime = 7.7;
	}
	if( !isDefined( fadeTime ) )
	{
		fadeTime = 1.3;
	}
	
	wait ( fadeTime );
	
	title = newClientHudElem( self );
	
	title.alpha = 0;
	title fadeOverTime( fadeTime );
	title.alpha = 1;
	
	if( direction == "left" )
	{
		title.x = -64;
		title.y = -24;	
	}
	else
	{
		title.x = 64;
		title.y = 0;
	}
		
	title titleTemplate();
		
	title setText( text );
	
	if( direction == "left" )
	{	
		title moveOverTime( moveTime );
		title.x = title.x - 12;
	}
	else
	{
		title moveOverTime( moveTime );
		title.x = title.x + 12;
	}
	
	wait ( moveTime - fadeTime );

	title fadeOverTime( fadeTime );
	title.alpha = 0;
		
	wait ( fadeTime );
	
	if( isDefined( title ) )
	{
		title destroy();
	}
}

titleTemplate()
{
	self.alignX = "center";
	self.alignY = "middle";
	self.horzAlign = "center";
	self.vertAlign = "middle";
	
	self.fontScale = 1.47;
	self.sort = 2;
}

/***********************************************************************
	ENTITIES
***********************************************************************/

deleteTargetname( name )
{
	ents = getEntArray( name, "targetname" );

	for( x = 0 ; x < ents.size ; x++ )
	{
		if( isDefined( ents[ x ] ) )
		{
			ents[ x ] delete();
		}
	}
	
}

deleteNoteworthy( name )
{
	ents = getEntArray( name, "script_noteworthy" );

	for( x = 0 ; x < ents.size; x++ )
	{
		if( isDefined( ents[ x ] ) )
		{
			ents[ x ] delete();	
		}
	}
}

/***********************************************************************
	AI / DRONE
***********************************************************************/

spawnDrone( location, angles, animName, weapon, canDie, health, animTree, idleAnims )
{
	guy = spawn( "script_model", groundpos( location ) );
	guy.angles = angles;
	
	backPack = RandomIntRange( 0, 1 );
	
	if ( backPack == 1 )
	{
		guy character\char_usa_marine_r_nb_rifle:: main();
	}
	else
	{
		guy character\char_usa_marine_r_rifle::main();
	}
	
	guy animTreeInit( animTree );
	
	guy.a = spawnStruct();
	guy.animName = animName;

	if( isDefined( weapon ) && weapon == 1 )
	{
			guy maps\_drones::drone_assign_weapon( "allies" );
	}

	guy.targetname = "drone";
	guy makeFakeAI();
	guy.team = "allies";
	
	if( !isDefined( health ) )
	{
		guy.health = 150;
	}
	else
	{
		guy.health = health;
	}
	
	guy.fakeDeath = true;
	if( isDefined( canDie ) && canDie == true )
	{
		guy thread maps\_drones::drone_fakeDeath();
	}
	
	guy.drone_run_cycle = level.drone_run_cycle[ "run_fast" ];
	
	if( isDefined( idleAnims ) )
	{
		guy thread droneIdleThink( idleAnims );
	}
	
	return guy;
}

animTreeInit( animTree )
{
	if( !isDefined( animTree ) )
	{
		self spawnDrone_GH();
	}
	else if( isDefined( animTree ) && animTree == "fakeshooters" )
	{
		self spawnDrone_FS();
	}
	else if( isDefined( animTree ) && animTree == "sniper_crows" )
	{
		self spawnDrone_SC();
	}
}

#using_animtree ( "generic_human" );
spawnDrone_GH()
{
	self UseAnimTree( #animtree );
}

#using_animtree ( "fakeshooters" );
spawnDrone_FS()
{
	self UseAnimTree( #animtree );
}

#using_animtree ( "sniper_crows" );
spawnDrone_SC()
{
	self UseAnimTree( #animtree );
}

droneIdleThink( idleAnim )
{
	self endon ( "death" );
	self endon ( "drone_stop" ); 
	
	idleAnim = idleAnim[ randomInt( idleAnim.size ) ];
	
	while( 1 )
	{
		self setFlaggedAnimKnob( "idle", idleAnim, 1, .1, 1 ); 
		
		self.a.coverMode = "hide"; 
	
		self waittillmatch( "idle", "end" ); 
	}
}

ignoremeOn()
{
	if( isAlive( self ) )
	{
		self.ignoreme = 1;
	}
}

ignoremeOff()
{
	if( isAlive( self ) )
	{
		self.ignoreme = 0;
	}
}

ignoreAllOn()
{
	if( isAlive( self ) )
	{
		self.ignoreall = true;
	}
}

ignoreAllOff()
{
	if( isAlive( self ) )
	{
		self.ignoreall = false;
	}
}

ignoreSuppressionOn()
{
	if( isAlive( self ) )
	{
		self.ignoresuppression = 1;
	}
}


ignoreSuppressionOff()
{
	if( isAlive( self ) )
	{
		self.ignoresuppression = 0;
	}
}

pacifistOn()
{
	if( isAlive( self ) )
	{
		self.pacifist = 1;
		self.pacifistwait = 0.05;
	}
}

pacifistOff()
{
	if( isAlive( self ) )
	{
		self.pacifist = 0;
		self.pacifistwait = 20;
	}
}

deleteDrones()
{
	drones = getEntArray( "drone", "targetname" );
	
	for( x = 0 ; x < drones.size ; x++ )
	{
		drones[ x ] maps\_drones::drone_delete();
	}
}